/*
 * Draw_Buffer_String_16x16.c
 *
 *  Created on: 2024/11/18
 *      Author: egaono-electronics
 */

#include "Draw_Buffer_String_16x16.h"

#ifdef def_USE_UTF16
#include def_H_Font_UTF16_16x16
#include "Calc_Address_Font_UTF16_16x16.h"
#endif  // def_USE_UTF16

#include "ILI9341_SPI_Draw.h"



//----------------------------------------------------------
// 描画Bufferに Font 16 x 16 文字をセット
//----------------------------------------------------------
// 引数;
// int16_t int16_Type :
//      0 : 通常表示
//      1 : 反転表示

// uint8_t *puint8_Address_font : フォント配列のポインタ

// uint16_t uint16_Length : 文字数
// uint16_t *puint16_aCode :
//                  文字コードが格納されたBufferのポインタ
// uint8_t *puint8_aBuffer : 描画Bufferのポインタ


// 戻り値:
//  なし
//----------------------------------------------------------
void Draw_Buffer_String_16x16(int16_t int16_Type,
                              uint8_t *puint8_Address_font,
                              uint16_t uint16_Length,
                              uint16_t *puint16_aCode,
                              uint8_t *puint8_aBuffer)
{
  uint32_t uint32_AddressFont;

  uint32_t uint32_Position;

  uint32_t uint32_I;
  uint32_t uint32_J;


  // 指定された文字数分、処理する。
  for(uint32_I = 0;
            uint32_I < (uint32_t)uint16_Length; uint32_I++){
    // フォントの位置を計算
#ifdef def_USE_UTF16
    if(puint8_Address_font == def_Font_UTF16_16x16_Array){
      uint32_AddressFont
            = Calc_Address_Font_UTF16_16x16(
                            puint16_aCode[uint32_I]);
    }
    else{
      uint32_AddressFont
            = (uint32_t)puint16_aCode[uint32_I] * 32;
    }
#else   // def_USE_UTF16
    uint32_AddressFont
            = (uint32_t)puint16_aCode[uint32_I] * 32;
#endif  // def_USE_UTF16

    // 1文字分のフォントデータをバッファにセットする。
    for(uint32_J = 0;
        uint32_J < (uint32_t)def_DotLengthwise_Font16x16;
        uint32_J++){
      // フォントデータをセットするバッファの位置を計算
      uint32_Position
            = (uint32_t)uint16_Length * 2 * uint32_J
            + uint32_I * 2;

      // バッファにフォントデータをセットする。
      if(int16_Type == 0){
        // 通常表示
      // 前半 8bits
        puint8_aBuffer[uint32_Position]
            = puint8_Address_font[
                    uint32_AddressFont + uint32_J * 2];
      // 後半 8bits
        puint8_aBuffer[uint32_Position + 1]
            = puint8_Address_font[
                    uint32_AddressFont + uint32_J * 2 + 1];
      }
      else{
        // 反転表示
      // 前半 8bits
        puint8_aBuffer[uint32_Position]
            = ~puint8_Address_font[
                    uint32_AddressFont + uint32_J * 2];
      // 後半 8bits
        puint8_aBuffer[uint32_Position + 1]
            = ~puint8_Address_font[
                    uint32_AddressFont + uint32_J * 2 + 1];
      }
    }
  }
}
